package reader.structure;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class ThreadTool {
	private List<Thread> threadList = new ArrayList<Thread>();

	public void addThread(Thread thread) {
		int index = this.threadList.indexOf(new Thread(thread.getId(), -1));
		if (index == -1) {
			this.threadList.add(thread);
		} else {
			this.threadList.get(index).setBegin(thread.getBegin());
		}
	}

	public void addBlock(Block block, String threadId) {
		int index = this.threadList.indexOf(new Thread(threadId, -1));
		if (block.getDuration() != 0L) {
			if (index == -1) {
				Thread thread = new Thread(threadId, -1);
				this.threadList.add(thread);
				thread.addBlock(block);
			} else {
				Thread thread = this.threadList.get(index);
				thread.addBlock(block);
			}
		}
	}

	public void adjustMutexes() {
		List<Block> mutexBlockList = new ArrayList<Block>();

		for (Thread thread : this.threadList) {
			for (Block block : thread.getBlockList()) {
				if (block.getBlockType() == BlockType.MUTEX) {
					mutexBlockList.add(block);
				}
			}
		}

		Collections.sort(mutexBlockList);

		for (int i = 1; i < mutexBlockList.size(); i++) {
			if (mutexBlockList.get(i).getBegin() < mutexBlockList.get(i - 1)
					.getEnd()) {
				Block mutexBlock = mutexBlockList.get(i);
				int index = mutexBlock.getThread().getBlockList().indexOf(
						mutexBlock);

				mutexBlock.getThread().putDelay(
						index,
						mutexBlockList.get(i - 1).getEnd()
								- mutexBlock.getBegin());
			}
		}
	}

	public void cleanWaitTimes() {
		for (Thread thread : this.threadList) {
			thread.cleanThread();
		}
	}

	public List<Thread> getThreadList() {
		return threadList;
	}
	
	public int getThreadListSize() {
		return this.getThreadList().size();
	}
	
	public long getTimeWaste() {
		long waste = 0;
		
		for(Thread thread : this.threadList) {
			for(Block block : thread.getBlockList()) {
				if (block.getBlockType() == BlockType.WAIT) {
					waste += block.getDuration();
				}
			}
		}
		
		return waste;
	}
}
